<?php
if (!defined('ABSPATH')) {
    exit;
}

class IOWD_Filter_Images
{
    private $options;

    /**
     * IOWD_Filter_Images constructor.
     *
     * @param $options
     */
    public function __construct($options)
    {
        $this->options = $options;
    }

    /**
     * @param $mime_type
     *
     * @return bool
     */
    public function check_optimization_level($mime_type)
    {
        $options = $this->options;

        switch (strtolower($mime_type)) {
            case "jpg":
            case "jpeg":
                if (!$options["jpg_optimization_levels"]) {
                    return false;
                }
                break;

            case "png":
                if (!$options["png_optimization_levels"]) {
                    return false;
                }
                break;

            case "gif":
                if (!$options["gif_optimization_levels"]) {
                    return false;
                }
                break;

            case "pdf":
                if (!$options["pdf_optimization_levels"]) {
                    return false;
                }
                break;
        }

        return true;
    }

    /**
     * @param $size
     *
     * @return bool
     */
    public function skip_small_images($size)
    {
        $options = $this->options;
        if ($size < floatval($options["skip_small_images"]) * 1048576 && $options["skip_small_images"]) {
            return false;
        }

        return true;
    }

    /**
     * @param $size
     *
     * @return bool
     */
    public function skip_large_images($size)
    {
        $options = $this->options;
        if ($size > floatval($options["skip_large_images"]) * 1048576 && $options["skip_large_images"]) {
            return false;
        }

        return true;
    }

    /**
     * @param $path
     *
     * @return string
     */
    public function get_current_opt_level($path)
    {
        $options = $this->options;
        $mime_type = strtolower(pathinfo($path, PATHINFO_EXTENSION));
        $mime_type = $mime_type == "jpeg" ? "jpg" : $mime_type;
        $optimization_type = isset($options[$mime_type . "_optimization_levels"]) ? $options[$mime_type . "_optimization_levels"] : "0";

        return $optimization_type;
    }
}