<?php
if (!defined('ABSPATH')) {
    exit;
}

class IOWD_Helper
{


    /**
     * @param $message
     * @param $type
     *
     * @return string
     */
    public static function message($message, $type)
    {
        return '<div class="iowd-messages">
				<div class="' . $type . '">
					<p><strong>' . $message . '</strong></p>
				</div>
			</div>';
    }

    /**
     * @param array  $url
     * @param string $page
     *
     * @return mixed
     */
    public static function redirect(array $url, $page = "admin.php")
    {
        return wp_safe_redirect(add_query_arg($url, admin_url($page)));
    }

    /**
     * @param      $dir_path
     * @param bool $only_dirs
     */
    public static function dir_tree($dir_path, $only_dirs = true)
    {

        if (file_exists($dir_path)) {
            echo "<ul>";
            $dir = scandir($dir_path);

            foreach ($dir as $file) {

                if ($file == "." || $file == ".." || strpos($file, ".") !== false) {
                    continue;
                }

                if (is_dir($dir_path . "/" . $file) || $only_dirs === false) {

                    echo '<li class="iowd-dir-tree-title" data-path="' . str_replace("\\", "/", $dir_path) . "/" . $file . '" ><a href="#" ><img src="' . IOWD_URL_IMG . '/folder.png" class="iowd-dir-tree-icon">' . $file . '</a>';
                }

                if (is_dir($dir_path . "/" . $file) === true) {
                    self::dir_tree($dir_path . "/" . $file, $only_dirs);
                }

            }
            echo "</li></ul>";
        }


    }

    /**
     * @param       $dir_path
     * @param array $image_paths
     *
     * @return array
     */
    public static function get_images_from_dir_recursiv($dir_path, &$image_paths = array())
    {
        $gallery_dir = IOWD_Helper::get_gallery_dir();
        $options = json_decode(get_option(IOWD_PREFIX . "_options"), true);

        if (file_exists($dir_path)) {
            $dir = scandir($dir_path);
            $images = glob($dir_path . '/*.{jpg,png,gif,jpeg,pdf}', GLOB_MARK | GLOB_BRACE);

            $images = $images ? $images : array();
            if ($images) {
                $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off' ? 'https' : 'http';
                array_walk($images, function (&$value) use ($protocol) {
                    $value = str_replace($_SERVER['DOCUMENT_ROOT'], $protocol . '://' . $_SERVER['SERVER_NAME'], $value);
                });
            }
            $image_paths = array_merge($image_paths, $images);
            foreach ($dir as $file) {

                if ($file == "." || $file == "..") {
                    continue;
                }

                if (!(strpos($file, ".original") !== false && $options["exclude_full_size"] == "0" && !empty($gallery_dir))) {
                    if ((strpos($file, ".") !== false)) {
                        continue;
                    }
                }

                if (is_dir($dir_path . "/" . $file)) {
                    self::get_images_from_dir_recursiv($dir_path . "/" . $file, $image_paths);
                }

            }
        }

        return $image_paths;

    }

    /**
     * @param $dir_path
     *
     * @return array
     */
    public static function get_images_from_dir($dir_path)
    {
        if (file_exists($dir_path)) {
            $images = glob($dir_path . '/*.{jpg,png,gif,jpeg,pdf}', GLOB_BRACE);

            $images = $images ? $images : array();
            if ($images) {
                $protocaol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off' ? 'https' : 'http';
                array_walk($images, create_function('&$value', '$value = str_replace( "' . $_SERVER['DOCUMENT_ROOT'] . '", "' . $protocaol . '://' . $_SERVER['SERVER_NAME'] . '", $value );'));
            }
        }

        return $images;

    }

    /**
     * @return array
     */
    public static function get_attachment_ids()
    {
        $args = array(
            'post_type'   => 'attachment',
            'numberposts' => -1
        );
        $attachments = get_posts($args);
        $images = array();

        foreach ($attachments as $attachment) {
            $images[] = $attachment->ID;
        }

        return $images;
    }

    /**
     * @return array
     */
    public static function wp_get_image_sizes()
    {
        $iowd_sizes = array();
        $sizes = get_intermediate_image_sizes();
        global $_wp_additional_image_sizes;
        foreach ($sizes as $size) {
            if ($size == 'thumbnail' || $size == 'medium' || $size == 'medium_large' || $size == 'large') {
                $iowd_sizes[$size]["width"] = get_option($size . '_size_w');
                $iowd_sizes[$size]["height"] = get_option($size . '_size_h');
            } else if (isset($_wp_additional_image_sizes[$size])) {
                $iowd_sizes[$size] = array(
                    'width'  => $_wp_additional_image_sizes[$size]['width'],
                    'height' => $_wp_additional_image_sizes[$size]['height'],
                );
            }
        }
        if ($iowd_sizes['medium_large']['width'] == 0) {
            $iowd_sizes['medium_large']['width'] = '768';
        }
        if ($iowd_sizes['medium_large']['height'] == 0) {
            $iowd_sizes['medium_large']['height'] = '9999';
        }

        return $iowd_sizes;
    }

    /**
     * @param string $attachment_url
     *
     * @return bool|integer
     */
    public static function pn_get_attachment_id_from_url($attachment_url = '')
    {

        global $wpdb;
        $attachment_id = false;

        // If there is no url, return.
        if ('' == $attachment_url)
            return false;

        // Get the upload directory paths
        $upload_dir_paths = wp_upload_dir();

        // Make sure the upload path base directory exists in the attachment URL, to verify that we're working with a media library image
        if (false !== strpos($attachment_url, $upload_dir_paths['baseurl'])) {

            // If this is the URL of an auto-generated thumbnail, get the URL of the original image
            $attachment_url = preg_replace('/-\d+x\d+(?=\.(jpg|jpeg|png|gif)$)/i', '', $attachment_url);

            // Remove the upload path base directory from the attachment URL
            $attachment_url = str_replace($upload_dir_paths['baseurl'] . '/', '', $attachment_url);

            // Finally, run a custom database query to get the attachment ID from the modified attachment URL
            $attachment_id = $wpdb->get_var($wpdb->prepare("SELECT wposts.ID FROM $wpdb->posts wposts, $wpdb->postmeta wpostmeta WHERE wposts.ID = wpostmeta.post_id AND wpostmeta.meta_key = '_wp_attached_file' AND wpostmeta.meta_value = '%s' AND wposts.post_type = 'attachment'", $attachment_url));

        }

        return $attachment_id;
    }

    /**
     * @param $url
     *
     * @return bool
     */
    public static function iowd_is_url_exists($url)
    {

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if ($code == 200) {
            $status = true;
        } else {
            $status = false;
        }
        curl_close($ch);

        return $status;
    }

    /**
     * @param $id
     *
     * @return array
     */
    public static function get_attachment_sizes($id)
    {
        $attachmnet_sizes = array();
        $full_file = get_attached_file($id);
        $full_file_size = file_exists($full_file) ? filesize($full_file) : 0;

        $meta = wp_get_attachment_metadata($id);
        $sizes = isset($meta["sizes"]) ? $meta["sizes"] : array();
        $attachmnet_sizes["full"] = $full_file_size;
        $attachment_root_dir = str_replace(basename($full_file), "", $full_file);
        $total = $full_file_size;
        foreach ($sizes as $size => $file) {
            $image_size = file_exists($attachment_root_dir . $file["file"]) ? filesize($attachment_root_dir . $file["file"]) : 0;
            $attachmnet_sizes[$size] = $image_size;
            $total += $image_size;

        }
        $attachmnet_sizes["total"] = $total;

        return $attachmnet_sizes;

    }

    /**
     * @param bool $refresh
     *
     * @return array
     */
    public static function limitation($refresh = false)
    {
        $already_optimized = get_site_transient("iowd_already_optimized");

        if (!(is_array($already_optimized) && count($already_optimized) == 5) || $refresh == true) {
            $already_optimized = self::update_already_used();
        }

        if (is_array($already_optimized) && count($already_optimized) == 5) {
            return array(
                "limit"             => $already_optimized["limit"],
                "period"            => $already_optimized["period"] . " months",
                "already_optimized" => $already_optimized['already_optimized'],
                "extreme_mode"      => $already_optimized["extreme_mode"],
                "auto_optimize"     => $already_optimized["auto_optimize"],

            );
        } else {
            return array(
                "limit"             => -1,
                "period"            => "1 month",
                "already_optimized" => 0,
                "extreme_mode"      => 0,
                "auto_optimize"     => 0,
            );
        }

    }

    /**
     * @return array
     */
    public static function update_already_used()
    {
        require_once IOWD_DIR_INCLUDES . "/iowd-api.php";
        IOWD_Api::set_api_action("used");
        $response = IOWD_Api::api_request();
        $already_optimized = isset($response["body"]) ? $response["body"] : array();

        set_site_transient("iowd_already_optimized", $already_optimized, 43200);

        return $already_optimized;
    }

    /**
     * @param bool $format_text
     *
     * @return array
     */
    public static function get_statistics($format_text = true)
    {
        global $wpdb;
        $rows = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "iowd_images");

        $total_size = 0;
        $total_orig_size = 0;
        $total_size_other = 0;
        $total_orig_size_other = 0;

        foreach ($rows as $row) {

            if ($row->media == 0) {
                $total_size_other += $row->image_size;
                $total_orig_size_other += $row->image_orig_size;
            } else {
                $total_size += $row->image_size;
                $total_orig_size += $row->image_orig_size;
            }

        }

        $total_reduced = $total_orig_size - $total_size;
        $total_reduced_percent = $total_orig_size ? ($total_reduced / $total_orig_size) * 100 : 0;

        $total_reduced_other = $total_orig_size_other - $total_size_other;
        $total_reduced_percent_other = $total_orig_size_other ? ($total_reduced_other / $total_orig_size_other) * 100 : 0;

        $total = $total_reduced + $total_reduced_other;
        $total_percent = ($total_orig_size + $total_orig_size_other) ? ($total / ($total_orig_size + $total_orig_size_other)) * 100 : 0;
        if ($format_text) {
            $total_reduced_percent = number_format($total_reduced_percent, 2);
            $total_reduced_percent_other = number_format($total_reduced_percent_other, 2);
            $total_percent = number_format($total_percent, 2);
        }

        return array(
            "total_reduced"               => $total_reduced,
            "total_reduced_percent"       => $total_reduced_percent,
            "total_reduced_other"         => $total_reduced_other,
            "total_reduced_percent_other" => $total_reduced_percent_other,
            "total"                       => $total,
            "total_percent"               => $total_percent,
        );
    }

    /**
     * @param array $settings
     */
    public static function sendSettings($settings = array())
    {
        $options = json_decode(get_option(IOWD_PREFIX . "_options"), true);
        $optimization_mode = isset($settings["optimization_mode"]) ? $settings["optimization_mode"] : get_option(IOWD_PREFIX . "_standart_setting");

        $enable_auto_optimization = isset($settings["enable_auto_optimization"]) ? $settings["enable_auto_optimization"] : $options["automatically_optimize"];
        $enable_photo_gallery_optimization = isset($settings["enable_photo_gallery_optimization"]) ? $settings["enable_photo_gallery_optimization"] : $options["optimize_gallery"];

        $enable_photo_gallery = isset($settings["photo_gallery_active"]) ? $settings["photo_gallery_active"] : (is_null(IOWD_Helper::get_gallery_dir()) ? 0 : 1);

        IOWD_Api::set_api_action("settings");
        IOWD_Api::set_post_data(array(
            "optimization_mode"                 => $optimization_mode,
            "enable_auto_optimization"          => $enable_auto_optimization,
            "enable_photo_gallery_optimization" => $enable_photo_gallery_optimization,
            "site_url"                          => site_url(),
            "photo_gallery_active"              => $enable_photo_gallery,
        ));
        IOWD_Api::api_request();
    }

    /**
     * @return null|string
     */
    public static function get_gallery_dir()
    {
        if (class_exists("BWG")) {
            $bwg = new BWG();
            $dir_path = str_replace(ABSPATH, '', $bwg->upload_dir);

            return ABSPATH . "/" . $dir_path;
        }

        return null;
    }

    public static function collect_settings()
    {
        $mode = get_option(IOWD_PREFIX . "_standart_setting") ? get_option(IOWD_PREFIX . "_standart_setting") : "balanced";
        $options = json_decode(get_option(IOWD_PREFIX . "_options"), true);
        $enable_photo_gallery = is_null(IOWD_Helper::get_gallery_dir()) ? 0 : 1;

        $data = array(
            "optimization_mode"                 => $mode,
            "enable_auto_optimization"          => $options["automatically_optimize"],
            "enable_photo_gallery_optimization" => $options["optimize_gallery"],
            "photo_gallery_active"              => $enable_photo_gallery,
        );

        return $data;
    }

    public static function send_settings_to_api($settings)
    {
        IOWD_Api::set_api_action("settings");
        IOWD_Api::set_post_data($settings);
        IOWD_Api::api_request("PATCH");
    }

    /**
     * function for creating db tables
     */
    public static function create_tables()
    {
        if (!get_option(IOWD_PREFIX . "_mode")) {
            add_option(IOWD_PREFIX . '_mode', "standart", '', 'no');
        }

        if (!get_option(IOWD_PREFIX . "_standart_setting")) {
            add_option(IOWD_PREFIX . '_standart_setting', "balanced", '', 'no');
        }

        self::add_options_to_db(false);
        require_once IOWD_DIR_CLASSES . "/iowddb.php";
        $db = new IOWDDB();
        $db->create_iowd_images_table();
        add_option(IOWD_PREFIX . "_tables_created", 1, '', 'no');
    }

    /**
     * @param bool $delete
     *
     * @return bool
     */
    public static function add_options_to_db($delete = false)
    {
        if ($delete == true) {
            delete_option(IOWD_PREFIX . "_options");
        } else {
            $options = get_option(IOWD_PREFIX . "_options");
            if ($options) {
                return false;
            }
        }
        require_once IOWD_DIR_INCLUDES . "/iowd-helper.php";
        $sizes = IOWD_Helper::wp_get_image_sizes();
        $sizes = is_array($sizes) && empty($sizes) === false ? implode(",", array_keys($sizes)) : "";

        $options = array(
            "api_key"                            => "",
            "automatically_optimize"             => "0",
            "keep_exif_data"                     => "1",
            "keep_originals"                     => "0",
            "jpg_optimization_levels"            => "lossy40",
            "png_optimization_levels"            => "lossy40",
            "gif_optimization_levels"            => "lossless",
            "pdf_optimization_levels"            => "0",
            "scheduled_optimization"             => "0",
            "scheduled_optimization_recurrence"  => "",
            "other_folders"                      => "",
            "resize_media_images_width"          => "",
            "resize_media_images_height"         => "",
            "enable_resizing"                    => "",
            "enable_resizing_other"              => "",
            "resize_other_images_width"          => "",
            "resize_other_images_height"         => "",
            "optimize_thumbs"                    => $sizes,
            "skip_small_images"                  => "",
            "skip_large_images"                  => "",
            "exclude_full_size"                  => "1",
            "exclude_full_size_metadata_removal" => "0",
            "optimize_once_more"                 => "1",
            "enable_conversion"                  => "0",
            "jpg_to_png"                         => "0",
            "png_to_jpg"                         => "0",
            "gif_to_png"                         => "0",
            "jpg_to_webp"                        => "0",
            "png_to_webp"                        => "0",
            "optimize_gallery"                   => "1",
        );

        $photo_gallery_dir = IOWD_Helper::get_gallery_dir();
        if ($photo_gallery_dir) {
            $images = IOWD_Helper::get_images_from_dir_recursiv(trim($photo_gallery_dir));
            if (!empty($images)) {
                $other_folders = array();
                $other_folders[$photo_gallery_dir] = $images;
                $options["other_folders"] = json_encode($other_folders);
            }
        }

        add_option(IOWD_PREFIX . "_options", json_encode($options), '', 'no');
    }


}