<?php
if (!defined('ABSPATH')) {
    exit;
}

class IOWD_Image_Processing
{
    private $options;

    /**
     * IOWD_Image_Processing constructor.
     *
     * @param $options
     */
    public function __construct($options)
    {
        $this->options = $options;
    }

    /**
     * @param       $item
     * @param array $meta
     *
     * @return array
     */
    public function resize($item, $meta = array())
    {
        $db = new IOWDDB_Class();
        $db->set_table_name("iowd_images");
        $options = json_decode(get_option(IOWD_PREFIX . "_options"), true);
        // do optional resizing and coversions
        if ($item) {
            $iowd_image = new IOWDImage();
            $row = $db->get_row_by_field(array("path" => $item["path"]));
            $resized = $row && $row->resized ? 1 : 0;
            $item["resized"] = $resized;

            if ($resized == 0) {
                $iowd_image->load_image($item["file"]);
                $flag = false;
                list($width) = @getimagesize($item["file"]);
                if ($item["media"] == 1 && $options["resize_media_images_width"] && $options["resize_media_images_height"] && $width > $options["resize_media_images_width"] && $options["enable_resizing"] == 1 && $item["size"] == "full") {
                    $iowd_image->resize($options["resize_media_images_width"], $options["resize_media_images_height"]);
                    $flag = true;
                } else if ($item["media"] == 0 && $options["resize_other_images_width"] && $options["resize_other_images_height"] && $width > $options["resize_other_images_width"] && $options["enable_resizing_other"] == 1) {
                    $iowd_image->resize($options["resize_other_images_width"], $options["resize_other_images_height"]);
                    $flag = true;
                }

                if ($flag == true) {
                    $item["resized"] = 1;
                    list($width, $height) = @getimagesize($item["file"]);
                    if (!$meta) {
                        $_wp_attachment_metadata = get_post_meta($item["post_id"], "_wp_attachment_metadata");
                        $_wp_attachment_metadata = $_wp_attachment_metadata[0];
                        $_wp_attachment_metadata["width"] = $width;
                        $_wp_attachment_metadata["height"] = $height;
                        update_post_meta($item["post_id"], "_wp_attachment_metadata", $_wp_attachment_metadata);
                    } else {
                        $meta["width"] = $width;
                        $meta["height"] = $height;
                    }
                }

            }

        }

        return array($item, $meta);
    }

    /**
     * @param $item
     *
     * @return mixed
     */
    public function convert(&$item)
    {
        require_once IOWD_DIR_CLASSES . "/iowdimage.php";
        require_once IOWD_DIR_INCLUDES . "/iowd-util.php";
        $options = json_decode(get_option(IOWD_PREFIX . "_options"), true);

        $iowd_image = new IOWDImage();
        $original_file = $item["file"];

        $mime_type = strtolower(pathinfo($item["file"], PATHINFO_EXTENSION));
        $flag = false;
        $iowd_image->load_image($original_file);
        if (($options["jpg_to_png"] == 1 && ($mime_type == "jpg" || $mime_type == "jpeg")) || ($options["gif_to_png"] == 1 && $mime_type == "gif")) {
            $iowd_image->convert_to_png();
            $flag = "png";

        } else if ($options["png_to_jpg"] == 1 && $mime_type == "png") {
            $iowd_image->convert_to_jpg();
            $flag = "jpg";
        } else if (($options["jpg_to_webp"] == 1 && ($mime_type == "jpg" || $mime_type == "jpeg")) || ($options["png_to_webp"] == 1 && $mime_type == "png")) {
            $iowd_image->convert_to_webp();
            $flag = "webp";

        }

        if ($flag) {
            $this->update_converted_data($flag, $item);
        }
        return $item;
    }

    /**
     * @param $flag
     * @param $item
     */
    private function update_converted_data($flag, &$item)
    {
        $original_file = $item["file"];
        $mime_type = strtolower(pathinfo($item["file"], PATHINFO_EXTENSION));
        $item["converted"] = 1;
        if ($item["media"] == 1) {
            if ($item["size"] == "full") {
                global $wpdb;
                // Update the post into the database
                $wpdb->query("UPDATE wp_posts SET guid = CONCAT(TRIM(TRAILING '" . $mime_type . "' FROM guid), '" . $flag . "'),

                         post_mime_type = 'image/" . $flag . "'  WHERE ID='" . $item["post_id"] . "'");


                // Update the post meta into the database
                $_wp_attached_file = get_post_meta($item["post_id"], "_wp_attached_file");
                $_wp_attached_file = $_wp_attached_file[0];
                $updated__wp_attached_file = IOWD_Util::str_lreplace($mime_type, $flag, $_wp_attached_file);
                update_post_meta($item["post_id"], "_wp_attached_file", $updated__wp_attached_file);

                $_wp_attachment_metadata = get_post_meta($item["post_id"], "_wp_attachment_metadata");
                $_wp_attachment_metadata = $_wp_attachment_metadata[0];
                $_wp_attachment_metadata["file"] = IOWD_Util::str_lreplace($mime_type, $flag, $_wp_attachment_metadata["file"]);
                update_post_meta($item["post_id"], "_wp_attachment_metadata", $_wp_attachment_metadata);

            } else {
                $_wp_attachment_metadata = get_post_meta($item["post_id"], "_wp_attachment_metadata");
                $_wp_attachment_metadata = $_wp_attachment_metadata[0];
                $_wp_attachment_metadata["sizes"][$item["size"]]["file"] = IOWD_Util::str_lreplace($mime_type, $flag, $_wp_attachment_metadata["sizes"][$item["size"]]["file"]);
                $_wp_attachment_metadata["sizes"][$item["size"]]["mime-type"] = "image/" . $flag;
                update_post_meta($item["post_id"], "_wp_attachment_metadata", $_wp_attachment_metadata);

            }
        }

        $image_name = str_replace("." . $mime_type, "", basename($item["file"]));
        $converted_path = str_replace(basename($item["path"]), "", $item["path"]) . $image_name . "." . $flag;

        $converted_file = str_replace(basename($item["file"]), "", $item["file"]) . $image_name . "." . $flag;

        $converted_image_size = filesize($converted_file);

        $item["converted_image_size"] = $converted_image_size;
        $item["path"] = $converted_path;

        unlink($original_file);
    }

}