<?php
if (!defined('ABSPATH')) {
    exit;
}


class IOWD_Scan
{
    private $options;

    /**
     * IOWD_Scan constructor.
     *
     */
    public function __construct()
    {
        $this->options = json_decode(get_option(IOWD_PREFIX . "_options"), true);
    }

    /**
     * @param $all_attachments
     *
     * @return array
     */
    public function scan($all_attachments)
    {
        global $wpdb;
        $options = $this->options;
        $_db_optimized_data = $wpdb->get_results("SELECT `path`, `post_id`, `size`, `already_optimized` FROM " . $wpdb->prefix . "iowd_images");

        // all data
        $media = isset($all_attachments["media"]) ? $all_attachments["media"] : array();
        $media_sizes = isset($all_attachments["media_sizes"]) ? $all_attachments["media_sizes"] : array();
        $other = isset($all_attachments["other"]) ? $all_attachments["other"] : array();
        $unique_attachments = isset($all_attachments["attachments"]) ? $all_attachments["attachments"] : array();

        $all_media_sizes = array_merge($media, $media_sizes);
        $all_data_sizes = array_merge($all_media_sizes, $other);
        $all_data = array_unique(array_merge($media, $other));

        $optimized_data_other = array();
        $optimized_data_media = array();
        $optimized_data_media_sizes = array();


        $db_optimized_data = array();
        if ($_db_optimized_data) {
            $filter_images = new IOWD_Filter_Images($this->options);
            foreach ($_db_optimized_data as $data) {
                $optimization_type = $filter_images->get_current_opt_level($data->path);
                $already_optimized = explode(",", $data->already_optimized);
                $already_optimized = array_unique($already_optimized);

                if ($options["optimize_once_more"] == "1" && (($optimization_type == "losselss" && empty($already_optimized) == true) || ($optimization_type == "lossy40" && ($already_optimized == array("lossless") || empty($already_optimized) == true)) || ($optimization_type == "lossy" && !in_array("lossy", $already_optimized)))) {
                    continue;
                }
                $post_id = $data->post_id;
                $path = $data->path;

                if (strpos($post_id, "iowd") !== false) {
                    $optimized_data_other[$post_id] = $path;
                } else {
                    if ($data->size == "full") {
                        $optimized_data_media[$post_id . "_" . $data->size] = $path;
                    }
                    $optimized_data_media_sizes[$post_id . "_" . $data->size] = $path;
                }
                $db_optimized_data[$post_id . "_" . $data->size] = $path;
            }

        }

        $optimized_data = array_merge($optimized_data_other, $optimized_data_media);

        $all_data_sizes = array_merge($all_data_sizes, $db_optimized_data);
        $all_data = array_merge($all_data, $optimized_data);


        return array(
            "optimized_data"                 => count($optimized_data),
            "optimized_data_sizes"           => count($db_optimized_data),
            "not_optimized_data_media"       => count($unique_attachments),
            "not_optimized_data_media_sizes" => count($all_media_sizes),
            "not_optimized_data_other"       => count($other),
            "all_data_sizes"                 => count($all_data_sizes),
            "all_data"                       => count($all_data),
        );
    }

    /**
     * @param  $scan_data
     *
     * @return array
     */
    public function scanTotal($scan_data = null)
    {
        // statistics data
        $statistics = IOWD_Helper::get_statistics(false);

        //get last optimized data
        $last_optimized_data = get_transient("iowd_last_optimized_data");
        $last_optimized_data = $last_optimized_data ? json_decode($last_optimized_data, true) : array("image_size" => 0, "image_orig_size" => 0, "image_count" => 0);

        $last_optimized_data_reduced = $last_optimized_data["image_orig_size"] - $last_optimized_data["image_size"];
        $last_optimized_data_reduced_percent = $last_optimized_data["image_orig_size"] ? ($last_optimized_data_reduced / $last_optimized_data["image_orig_size"]) * 100 : 0;

        //scan data
        if (empty($scan_data)) {
            $scan_data = get_transient("iowd_scan_data_cache");
        }

        return array(
            "media_reduced"                       => $statistics["total_reduced"],
            "media_reduced_percent"               => $statistics["total_reduced_percent"],
            "other_reduced"                       => $statistics["total_reduced_other"],
            "other_reduced_percent"               => $statistics["total_reduced_percent_other"],
            "total_reduced"                       => $statistics["total"],
            "total_reduced_percent"               => $statistics["total_percent"],
            "last_optimized_reduced"              => $last_optimized_data_reduced,
            "last_optimized_reduced_percent"      => $last_optimized_data_reduced_percent,
            "not_optimized_data_media_with_sizes" => $scan_data["not_optimized_data_media_sizes"],
            "not_optimized_data_media"            => $scan_data["not_optimized_data_media"],
            "not_optimized_data_other"            => $scan_data["not_optimized_data_other"],
            "total_images_count"                  => $scan_data["all_data_sizes"],
            "total_optimized_count"               => $scan_data["optimized_data_sizes"],
        );
    }

    public function update_scan_cache($limit = 0)
    {
        $scan_url = add_query_arg(array('rest_route' => '/iowd/v1/scan'), get_home_url() . "/");
        $scan_all_url = add_query_arg(array('rest_route' => '/iowd/v1/log'), get_home_url() . "/");

        $scan_response = wp_remote_post($scan_url, array(
                'method'  => 'POST',
                'timeout' => 50000,
                'body'    => array(
                    "nonce_iowd" => wp_create_nonce('nonce_' . IOWD_PREFIX),
                    "limit"      => $limit,
                )
            )
        );
        if (!is_wp_error($scan_response) || wp_remote_retrieve_response_code($scan_response) === 200) {
            if (isset($scan_response["body"])) {
                $body = json_decode($scan_response['body'], true);
                if ($body["status"] != "done") {
                    $limit = $limit + 2000;
                    $this->update_scan_cache($limit);
                } else {
                    $scan_total = wp_remote_post($scan_all_url, array(
                            'method'  => 'POST',
                            'timeout' => 50000,
                            'body'    => array(
                                "nonce_iowd" => wp_create_nonce('nonce_' . IOWD_PREFIX),
                                "limit"      => $limit
                            )
                        )
                    );

                    if (!is_wp_error($scan_total) && isset($scan_total["body"])) {
                        return json_decode($scan_total['body'], true);
                    }
                }
            }
        }

        return false;

    }
}