<?php

if (!defined('ABSPATH')) {
    exit;
}

class IOWD
{
    protected static $instance = null;
    private static $version = '5.0.36';
    private static $page;
    private $reg_autoloader = false;
    private $options = array();

    /**
     * IOWD constructor.
     */
    private function __construct()
    {
        //ini_set('memory_limit', '-1');

        self::$page = isset($_GET["page"]) ? $_GET["page"] : 'iowd';
        $this->options = json_decode(get_option(IOWD_PREFIX . "_options"), true);

        // add_action('admin_post_save_settings', array('IOWD_Settings', 'save_settings'));
        // add_action('admin_post_nopriv_save_settings', array('IOWD_Settings', 'save_settings'));
        add_action('admin_init', array('IOWD_Settings', 'save_settings'));
        add_action('admin_init', array('IOWD_Logs', 'clear_logs'));

        // ajax
        add_action('wp_ajax_choose_dirs', array($this, 'ajax'));
        add_action('wp_ajax_optimize', array($this, 'ajax'));
        add_action('wp_ajax_nopriv_optimize', array($this, 'ajax'));
        add_action('wp_ajax_get_attachment_data', array($this, 'ajax'));
        add_action('wp_ajax_finish_bulk', array($this, 'ajax'));
        add_action('wp_ajax_abort', array($this, 'ajax'));

        add_action('wp_ajax_filter_report', array($this, 'ajax'));
        add_action('wp_ajax_get_stats', array($this, 'ajax'));
        add_action('wp_ajax_clear_report', array($this, 'ajax'));
        add_action('wp_ajax_quick_settings', array($this, 'ajax'));
        add_action('wp_ajax_update_already_used', array($this, 'ajax'));

        add_action('admin_notices', array($this, 'notice'));

        add_action('wp_ajax_get_subdirs', array($this, 'ajax'));

        // autoloder
        add_action('init', array($this, 'register_autoloader'));

        require_once IOWD_DIR_INCLUDES . "/iowd-helper.php";
        $limitation = IOWD_Helper::limitation();
        if ($limitation["already_optimized"] < $limitation["limit"] && !empty($limitation["auto_optimize"])) {
            add_filter('wp_update_attachment_metadata', array($this, 'auto_optimize'), 15, 2);
        }

        add_action('admin_init', array($this, 'actions'));

        // Add menu
        add_action('admin_menu', array($this, 'admin_menu'));

        // Add admin styles and scripts
        add_action('admin_enqueue_scripts', array($this, 'styles'));
        add_action('admin_enqueue_scripts', array($this, 'scripts'));

        add_action('admin_init', array($this, 'includes'));

        if (defined('DOING_CRON') && DOING_CRON) {
            add_action('iowd_optimize', array($this, 'scheduled_optimization'));
        }

        if (!wp_next_scheduled('iowd_optimize') && $this->options["scheduled_optimization"] == 1 && $limitation["already_optimized"] < $limitation["limit"]) {

            wp_schedule_event(time(), $this->options["scheduled_optimization_recurrence"], 'iowd_optimize');
        }
        if (!get_option(IOWD_PREFIX . "_tables_created")) {
            IOWD_Helper::create_tables();
        }
        add_action('wpmu_new_blog', array($this, 'new_blog_added'), 10, 6);

    }

    public static function activate()
    {
        if (function_exists('is_multisite') && is_multisite()) {
            global $wpdb;
            // Get all blog ids.
            $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                self::image_optimizer_activate();
                restore_current_blog();
            }

            return;
        } else {
            self::image_optimizer_activate();
        }
    }

    public function new_blog_added($blog_id)
    {
        if (!function_exists('is_plugin_active_for_network')) {
            require_once(ABSPATH . '/wp-admin/includes/plugin.php');
        }
        if (is_plugin_active_for_network(IOWD_MAIN_FILE)) {
            switch_to_blog($blog_id);
            self::image_optimizer_activate();
            restore_current_blog();
        }
    }

    /**
     * plugin activation function
     */
    public static function image_optimizer_activate()
    {
        //delete_transient('iowd_update_check');
        require_once IOWD_DIR_INCLUDES . "/iowd-helper.php";
        IOWD_Helper::update_already_used();
        $version = get_option(IOWD_PREFIX . "_version");
        if (get_option(IOWD_PREFIX . "_pro")) {
            update_option(IOWD_PREFIX . "_pro", "yes");
        } else {
            add_option(IOWD_PREFIX . "_pro", "yes", '', 'no');
        }

        if ($version && version_compare($version, self::$version, '<')) {
            update_option(IOWD_PREFIX . "_version", self::$version);

        } else {
            add_option(IOWD_PREFIX . "_version", self::$version, '', 'no');

            IOWD_Helper::create_tables();
        }

    }


    /**
     * plugin deactivation function
     */
    public static function deactivate()
    {
        wp_clear_scheduled_hook('iowd_optimize');
    }

    /**
     * @return IOWD|null
     */
    public static function get_instance()
    {
        if (null == self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * @param $class
     */
    public static function autoload($class)
    {
        $class = str_replace("_", "-", strtolower($class));
        $include_file = IOWD_DIR_INCLUDES . "/" . $class . ".php";
        if (file_exists($include_file)) {
            include_once $include_file;
        }
    }

    /**
     * @param $class
     */
    public static function autoloadProxy($class)
    {
        __autoload($class);
    }

    /**
     * admin notices
     */
    public function notice()
    {
        if ($this->is_iowd_page()) {
            $whitelist = array(
                '127.0.0.1',
                '::1'
            );
            if (in_array($_SERVER['REMOTE_ADDR'], $whitelist)) {
                echo "<div class='error'><p>" . __("Image optimizing is disabled on Localhost. Please install the plugin on a live server to optimize images.", IOWD_PREFIX) . "</p></div>";
            }
            $limitation = IOWD_Helper::limitation();
            if (!get_site_option(IOWD_MANAGER_PREFIX . '_access_token')) {
                echo "<div class='error'><p>" . __("To optimize your images, please log in to 10Web Manager.", IOWD_PREFIX) . "</p></div>";
            } else if ($limitation["already_optimized"] >= $limitation["limit"]) {
                $limit = $limitation["limit"] == -1 ? 0 : $limitation["limit"];
                echo "<div class='error'><p>" . sprintf(__("Your subscription plan allows optimizing %s images per month. This limitation has expired for current month.", IOWD_PREFIX), $limit) . "</p></div>";
            }
            if (!class_exists("WP_REST_Controller")) {
                echo "<div class='error'><p>" . __("Image Optimizer plugin requires WordPress 4.7 or higher.", IOWD_PREFIX) . "</p></div>";
            }
        }
    }

    /**
     * function for actions
     */
    public function actions()
    {

        if (isset($_GET["iowd_mode"])) {
            $mode = ($_GET["iowd_mode"] == "standart" || $_GET["iowd_mode"] == "advanced") ? $_GET["iowd_mode"] : "standart";

            if (get_option(IOWD_PREFIX . "_mode")) {
                update_option(IOWD_PREFIX . "_mode", $mode);
            } else {
                add_option(IOWD_PREFIX . '_mode', $mode, '', 'no');
            }
            if ($mode == "standart") {
                $standart_setting = get_option(IOWD_PREFIX . "_standart_setting");
                $settings = json_decode(get_option(IOWD_PREFIX . "_options"), true);
                if ($standart_setting == "conservative") {
                    $settings["keep_exif_data"] = 1;
                    $settings["exclude_full_size_metadata_removal"] = 1;
                    $settings["exclude_full_size"] = 1;
                    $settings["jpg_optimization_levels"] = "lossless";
                    $settings["png_optimization_levels"] = "lossless";
                    $settings["gif_optimization_levels"] = "lossless";
                    $settings["pdf_optimization_levels"] = "lossless";

                    update_option(IOWD_PREFIX . "_options", json_encode($settings));
                } else if ($standart_setting == "balanced") {
                    $settings["keep_exif_data"] = 1;
                    $settings["exclude_full_size_metadata_removal"] = 1;
                    $settings["exclude_full_size"] = 1;
                    $settings["jpg_optimization_levels"] = "lossy40";
                    $settings["png_optimization_levels"] = "lossy40";
                    $settings["gif_optimization_levels"] = "lossless";
                    $settings["pdf_optimization_levels"] = "lossless";

                    update_option(IOWD_PREFIX . "_options", json_encode($settings));
                } else if ($standart_setting == "extreme") {
                    $settings["keep_exif_data"] = 0;
                    $settings["exclude_full_size_metadata_removal"] = 1;
                    $settings["exclude_full_size"] = 1;
                    $settings["jpg_optimization_levels"] = "lossy";
                    $settings["png_optimization_levels"] = "lossy";
                    $settings["gif_optimization_levels"] = "lossless";
                    $settings["pdf_optimization_levels"] = "lossless";

                    update_option(IOWD_PREFIX . "_options", json_encode($settings));
                }

            }

            IOWD_Helper::redirect(array("page" => "iowd_settings"));
        }

    }

    /**
     * @param $meta
     * @param $id
     *
     * @return array
     */
    public function auto_optimize($meta, $id)
    {
        $options = $this->options;
        if ($options["automatically_optimize"] == 0) {
            return $meta;
        }
        $optimize = new IOWD_Optimize(true);
        $ids = is_array($id) ? $id : array($id);

        $optimize->get_temp_data($ids, false, true, $meta);
        $meta = $optimize->optimize(0, "auto_" . $id, 1, $meta);

        return $meta;
    }

    /**
     * function for scheduled optimization
     */
    public function scheduled_optimization()
    {
        $options = $this->options;

        if ($options["scheduled_optimization"] == 0) {
            return;
        }
        $optimize = new IOWD_Optimize();
        $data_count = $optimize->get_temp_data(array(), true);
        for ($i = 0; $i < $data_count; $i++) {
            $optimize->optimize($i);
        }

    }

    /**
     * admin menu function
     */
    public function admin_menu()
    {
        add_menu_page("Image Optimizer", "Image Optimizer", 'manage_options', 'iowd_settings', array($this, 'iowd_admin'), IOWD_URL_IMG . "/icon.png");

        add_submenu_page('iowd_settings', 'Image Optimizer', 'Report', 'manage_options', 'iowd_report', array($this, 'iowd_admin'));
        add_submenu_page(null, 'Logs', 'Report', 'manage_options', 'iowd_logs', array($this, 'iowd_admin'));
    }

    /**
     * function for displaying admin pages
     */
    public function iowd_admin()
    {
        if ($this->is_iowd_page() == true) {
            $this->user_manual();
            $view_class = ucfirst(strtolower(self::$page));
            $view = new $view_class();
            $view->display();
        }
    }

    /**
     * @return bool
     */
    private function is_iowd_page()
    {
        $iowd_pages = array(
            IOWD_PREFIX . "_settings",
            IOWD_PREFIX . "_report",
            IOWD_PREFIX . "_logs",
        );

        if (in_array(self::$page, $iowd_pages) == true) {
            return true;
        }

        return false;

    }

    /**
     * function for ajax requests
     */
    public function ajax()
    {
        check_ajax_referer('nonce_' . IOWD_PREFIX, 'nonce_' . IOWD_PREFIX);
        $action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : '';

        if ($action) {
            if (method_exists('IOWD_Ajax', $action)) {
                IOWD_Ajax::$action();
            }
        }
        wp_die();

    }

    /**
     * register autoloader function
     */
    public function register_autoloader()
    {
        if ($this->reg_autoloader) {
            return;
        }
        spl_autoload_register(array(__CLASS__, "autoload"));

        if (function_exists("__autoload")) {
            spl_autoload_register(array(__CLASS__, "autoloadProxy"));
        }

        $this->reg_autoloader = true;
    }

    /**
     * function for library includes
     */
    public function includes()
    {
        require_once IOWD_DIR_CLASSES . "/iowddb.php";
        require_once IOWD_DIR_CLASSES . "/iowdimage.php";
    }

    /**
     * function for admin css styles
     */
    public function styles()
    {
        wp_admin_css('thickbox');
        if ($this->is_iowd_page() == true) {
            wp_enqueue_style(IOWD_PREFIX . '_admin_main-css', IOWD_URL_CSS . '/admin_main.css', array(), self::$version);
            wp_enqueue_style(IOWD_PREFIX . '_calendar-css', IOWD_URL_CSS . '/calendar-jos.css');
        }

    }

    /**
     * function for admin js scripts
     */
    public function scripts()
    {
        $options = $this->options;
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-ui');
        wp_enqueue_script('jquery-ui-tooltip');

        // api js
        wp_enqueue_script(IOWD_PREFIX . '_api-js', IOWD_URL_JS . '/api.js', array(), self::$version);
        wp_localize_script(IOWD_PREFIX . '_api-js', 'apiText', array(
            "error_1" => __("Invalid Api key, or Api key not found.", IOWD_PREFIX),

        ));

        // admin main js
        wp_enqueue_script(IOWD_PREFIX . '_admin_main-js', IOWD_URL_JS . '/admin_main.js', array(), self::$version);

        global $pagenow;
        $is_media_page = ($pagenow == 'upload.php' || $pagenow == 'post.php') && !(isset($_GET["page"]) && $_GET["page"] == "iowd_settings") ? 1 : 0;

        wp_localize_script(IOWD_PREFIX . '_admin_main-js', 'iowd', array(
            "options"               => get_option(IOWD_PREFIX . "_options", array()),
            "finishUploadingBulk"   => __(" images are ready to be optimized.", IOWD_PREFIX),
            "finishUploadingSingle" => __("Processing optimize", IOWD_PREFIX),
            "ajaxURL"               => admin_url('admin-ajax.php'),
            "nonce"                 => wp_create_nonce('nonce_' . IOWD_PREFIX),
            "page"                  => self::$page,
            "iowd_active_tab"       => (isset($_REQUEST["iowd_tabs_active"]) ? $_REQUEST["iowd_tabs_active"] : ""),
            "is_media_page"         => $is_media_page,
            "iowd_image_url"        => IOWD_URL_IMG,
            "enable_resizing"       => $options["enable_resizing"],
            "enable_resizing_other" => $options["enable_resizing_other"],
            "iowd_optimizing"       => (isset($_GET["iowd_optimizing"]) ? $_GET["iowd_optimizing"] : 0),
            "scan_url"              => add_query_arg(array('rest_route' => '/iowd/v1/scan'), get_home_url() . "/"),
            "scan_all_url"          => add_query_arg(array('rest_route' => '/iowd/v1/log'), get_home_url() . "/")

        ));


        if ($this->is_iowd_page() == true) {

            wp_enqueue_script(IOWD_PREFIX . '_settings-js', IOWD_URL_JS . '/settings.js', array(), self::$version);

            $from_gallery = ((isset($_GET["target"]) && $_GET["target"] == "wd_gallery")
                || (empty(get_option("iowd_check_gallery")) && IOWD_Helper::get_gallery_dir())) ? 1 : 0;

            wp_localize_script(IOWD_PREFIX . '_settings-js', 'iowdSettingsGlobal', array(
                "image_url"             => IOWD_URL_IMG,
                "ajaxURL"               => admin_url('admin-ajax.php'),
                "nonce"                 => wp_create_nonce('nonce_' . IOWD_PREFIX),
                "page"                  => (isset($_GET["page"]) ? $_GET["page"] : IOWD_PREFIX . "_settings"),
                "save_dirs_txt"         => __("Save directories", IOWD_PREFIX),
                "save_gallery_dirs_txt" => __("Save Gallery directories", IOWD_PREFIX),
                "from_gallery"          => $from_gallery,
                "limitation"            => IOWD_Helper::limitation(),
            ));

            wp_enqueue_script(IOWD_PREFIX . '_calendar-js', IOWD_URL_JS . '/calendar.js', array(), self::$version, true);
            wp_enqueue_script(IOWD_PREFIX . '_calendar_function-js', IOWD_URL_JS . '/calendar_function.js', array(), self::$version, true);

            if (IOWD_Helper::get_gallery_dir()) {
                add_option("iowd_check_gallery", 1);
            }
        }

    }

    /**
     * function for admin banner
     */
    public function user_manual()
    {
        $page = isset($_GET["page"]) ? $_GET["page"] : "";
        ?>
        <div class="iowd_upgrade wd-clear">
            <div class="wd-left">
                <?php
                switch ($page) {
                    case "iowd_settings": ?>
                        <div style="font-size: 14px;margin-top: 6px;">
                            <?php
                            if (get_option(IOWD_PREFIX . '_mode') == "standart") {
                                _e("This section allows you quickly optimize the pictures, without going through advanced settings.", IOWD_PREFIX); ?>
                                <a style="color: #5CAEBD; text-decoration: none;border-bottom: 1px dotted;"
                                   target="_blank"
                                   href="http://docs.10web.io/docs/image-optimizer-wd/easy-mode.html"><?php _e("Read More in User Manual.", IOWD_PREFIX); ?></a>
                                <?php
                            } else {
                                _e("This section allows you configure image optimization and set up what best fits your website.", IOWD_PREFIX); ?>
                                <a style="color: #5CAEBD; text-decoration: none;border-bottom: 1px dotted;"
                                   target="_blank"
                                   href="http://docs.10web.io/docs/image-optimizer-wd/advanced-mode"><?php _e("Read More in User Manual.", IOWD_PREFIX); ?></a>
                                <?php
                            }
                            ?>
                        </div>
                        <?php
                        break;
                    case "iowd_report":
                        ?>
                        <div style="font-size: 14px;margin-top: 6px;">
                            <?php _e("This section allows you to check the optimization results.", IOWD_PREFIX); ?>
                            <a style="color: #5CAEBD; text-decoration: none;border-bottom: 1px dotted;" target="_blank"
                               href="http://docs.10web.io/docs/image-optimizer-wd/statistics-report"><?php _e("Read More in User Manual.", IOWD_PREFIX); ?></a>
                        </div>
                        <?php
                        break;
                }
                ?>
            </div>
        </div>
        <?php
    }
}



